import 'package:crypto/crypto.dart';
import 'package:ngx_core/ngx_core.dart';
import 'package:test/test.dart';

/// Tests the features of the [Security] class.
void main() => group('Security', () {
      const md5Hash = 'de2a7edc99adfbcc8c5b4c20b8b17a50';
      const sha1Hash = 'bff82d4577ec27502ccd743f3bbf706a0bf82564';
      const sha256Hash =
          '77853caf12f9d2020f3715accd9c7b15738705daf03650906af17a7a655c2ee2';

      group('.generatePasswordHash()', () {
        test('should generate MD5 password hashes', () {
          final security = Security()..passwordHashStrategy = md5;
          expect(security.generatePasswordHash('FooBar123'), md5Hash);
        });

        test('should generate SHA-1 password hashes', () async {
          final security = Security()..passwordHashStrategy = sha1;
          expect(security.generatePasswordHash('FooBar123'), sha1Hash);
        });

        test('should generate SHA-256 password hashes', () async {
          final security = Security()..passwordHashStrategy = sha256;
          expect(security.generatePasswordHash('FooBar123'), sha256Hash);
        });
      });

      group('.generateRandomKey()', () {
        test('should generate a random byte array of the specified length', () {
          final security = Security();
          expect(security.generateRandomKey(16), hasLength(16));
          expect(security.generateRandomKey(64), hasLength(64));
          expect(security.generateRandomKey(256), hasLength(256));
        });
      });

      group('.generateRandomString()', () {
        test('should generate a random string of the specified length', () {
          final security = Security();
          expect(security.generateRandomString(16), hasLength(16));
          expect(security.generateRandomString(64), hasLength(64));
          expect(security.generateRandomString(256), hasLength(256));
        });

        test(
            'should generate random strings only containing "base64url" characters',
            () {
          final base64urlChars = RegExp(r'^[a-z\d\-_]+$', caseSensitive: false);
          final security = Security();
          expect(security.generateRandomString(16), matches(base64urlChars));
          expect(security.generateRandomString(64), matches(base64urlChars));
          expect(security.generateRandomString(256), matches(base64urlChars));
        });
      });

      group('.validatePassword()', () {
        test('should return `true` for a valid password encoded in MD5', () {
          final security = Security()..passwordHashStrategy = md5;
          expect(security.validatePassword('FooBar123', md5Hash), isTrue);
        });

        test('should return `false` for an invalid password encoded in MD5',
            () {
          final security = Security()..passwordHashStrategy = md5;
          expect(security.validatePassword('foo_bar_123', md5Hash), isFalse);
        });

        test('should return `true` for a valid password encoded in SHA-1', () {
          final security = Security()..passwordHashStrategy = sha1;
          expect(security.validatePassword('FooBar123', sha1Hash), isTrue);
        });

        test('should return `false` for an invalid password encoded in SHA-1',
            () {
          final security = Security()..passwordHashStrategy = sha1;
          expect(security.validatePassword('foo_bar_123', sha1Hash), isFalse);
        });

        test('should return `true` for a valid password encoded in SHA-256',
            () {
          final security = Security()..passwordHashStrategy = sha256;
          expect(security.validatePassword('FooBar123', sha256Hash), isTrue);
        });

        test('should return `false` for an invalid password encoded in SHA-256',
            () {
          final security = Security()..passwordHashStrategy = sha256;
          expect(security.validatePassword('foo_bar_123', sha256Hash), isFalse);
        });
      });
    });
